# Code for the BCSS tutorial on using JAGS for wildlife data analysis
# https://bcss.org.my/tut/bayes-with-jags-a-tutorial-for-wildlife-researchers/occupancy-modelling/two-species-occupancy-models/
# Author: Mike Meredith
# License: GPL 3
# ----------------------------------------------------------------------------

# Exploring the occ2sps function and railSims data set from wiqid

# Using the 'nimble' package

library(nimble)
library(wiqid)

# Get the data
?railSims
data(railSims)
head(railSims)
#   A1 A2 A3 B1 B2 B3 logArea reeds
# 1  0  0  0  1  1  1  -0.585  TRUE
# 2  1  1  1  0  1  0  -0.231 FALSE
# 3  1  0  0  0  1  1   1.686 FALSE
# 4  0  0  0  1  1  0   0.092  TRUE
# 5  0  0  0  1  1  1   0.155 FALSE
# 6  1  1  1  0  0  0   1.854 FALSE

# Separate the two detection histories
DHA <- as.matrix(railSims[, 1:3])
DHB <- as.matrix(railSims[, 4:6])

# The model with occupancy interaction
# ====================================

# Write the model for NIMBLE
occ2sps_Int.code <- nimbleCode({
  # Likelihood
  for(i in 1:nSites) {
    # biological model
    zA[i] ~ dbern(psiA)
    zB[i] ~ dbern(psiB[zA[i] + 1])
    # observation model
    yA[i] ~ dbin(pA * zA[i], n)
    yB[i] ~ dbin(pB * zB[i], n)
  }

  # Priors
  pA ~ dbeta(1, 1) # Uninformative priors
  pB ~ dbeta(1, 1)
  psiA ~ dbeta(1, 1)
  psiB[1] ~ dbeta(1, 1)
  psiB[2] ~ dbeta(1, 1)
})

# Aggregate detection data
yA <- rowSums(DHA)
yB <- rowSums(DHB)

nSites <- nrow(railSims)
zA <- ifelse(yA > 0, 1, NA)
zB <- ifelse(yB > 0, 1, NA)

nimData <- list(yA = yA, yB = yB, zA = zA, zB = zB)
nimConst <- list(n = 3, nSites = nSites)

wanted <- c("pA", "pB", "psiA", "psiB")

system.time(
out1 <- nimbleMCMC(occ2sps_Int.code, nimConst, nimData, monitors=wanted,
    nchains=3, niter=11e3, nburnin=1e3, samplesAsCodaMCMC=TRUE) ) # 40 secs
# Works fine, but lots of ugly warnings:
#   dynamic index out of bounds: dbern(size = 1, prob = psiB[zA[i] + 1])
# Caused when initial value of zA = NA; avoid this by providing inits for zA.
diagPlot(out1)  # convergence is good

# Provide inits for zA
inits <- function() {
  zAinit <- rbinom(nSites, 1, 0.5)  # random 0/1
  zAinit[!is.na(zA)] <- NA          # no init if zA is known
  list(zA = zAinit)
}

system.time(
out1 <- nimbleMCMC(occ2sps_Int.code, nimConst, nimData, inits, monitors=wanted,
    nchains=3, niter=11e3, nburnin=1e3, samplesAsCodaMCMC=TRUE) ) # 40 secs
# No more warnings :-)
diagPlot(out1)  # convergence is good

( mc2sps1 <- mcmcOutput(out1) )
summary(mc2sps1)
#          mean    sd median   l95   u95  Rhat MCEpc
# pA      0.796 0.028  0.797 0.742 0.852 1.000 0.797
# pB      0.717 0.029  0.717 0.657 0.770 1.000 0.846
# psiA    0.467 0.040  0.467 0.389 0.543 1.000 0.564
# psiB[1] 0.818 0.045  0.821 0.729 0.902 0.999 0.690
# psiB[2] 0.391 0.057  0.390 0.279 0.502 1.000 0.605

postPlot(mc2sps1)
postPlot(mc2sps1, "psiB", xlim=c(0.25, 0.95))

# Interaction and detection
# =========================

occ2sps_Det.code <- nimbleCode({
  # Likelihood
  for(i in 1:nSites) {
    # biological model
    zA[i] ~ dbern(psiA)
    zB[i] ~ dbern(psiB[zA[i] + 1])
    # observation model
    for(j in 1:n) {
      DHA[i, j] ~ dbern(pA[zB[i] + 1] * zA[i])             # pA has 2 values
      DHB[i, j] ~ dbern(pB[zA[i] + DHA[i, j] + 1] * zB[i]) # pB has 3 values
    }
  }

  # Priors
  pA[1] ~ dbeta(1, 1) # spsB absent
  pA[2] ~ dbeta(1, 1) # spsB present
  pB[1] ~ dbeta(1, 1) # spsA absent
  pB[2] ~ dbeta(1, 1) # spsA present, not detected
  pB[3] ~ dbeta(1, 1) # spsA present, detected
  psiA ~ dbeta(1, 1)
  psiB[1] ~ dbeta(1, 1)
  psiB[2] ~ dbeta(1, 1)
})

nimData <- list(DHA = DHA, DHB = DHB, zA = zA, zB = zB)
nimConst <- list(n = 3, nSites = nSites)

# Provide inits for zA and zB
inits <- function() {
  zAinit <- rbinom(nSites, 1, 0.5)  # random 0/1
  zAinit[!is.na(zA)] <- NA          # no init if zA is known
  zBinit <- rbinom(nSites, 1, 0.5)  # random 0/1
  zBinit[!is.na(zB)] <- NA          # no init if zB is known
  list(zA = zAinit, zB = zBinit)
}

system.time(
out2 <- nimbleMCMC(occ2sps_Det.code, nimConst, nimData, inits, monitors=wanted,
    nchains=3, niter=11e3, nburnin=1e3, samplesAsCodaMCMC=TRUE) ) # 60 secs
diagPlot(out2)  # convergence is good

( mc2sps2 <- mcmcOutput(out2) )
summary(mc2sps2)
#          mean    sd median   l95   u95  Rhat MCEpc
# pA[1]   0.846 0.039  0.848 0.766 0.919 1.000 1.015
# pA[2]   0.729 0.053  0.732 0.623 0.829 1.000 1.129
# pB[1]   0.809 0.029  0.810 0.750 0.864 1.001 0.834
# pB[2]   0.679 0.104  0.685 0.482 0.881 1.000 1.156
# pB[3]   0.350 0.069  0.349 0.218 0.486 0.999 1.271
# psiA    0.469 0.040  0.469 0.392 0.547 1.001 0.579
# psiB[1] 0.802 0.044  0.804 0.716 0.885 1.000 0.531
# psiB[2] 0.481 0.081  0.476 0.331 0.646 1.000 1.403

plot(mc2sps2)
plot(mc2sps2, "pA", xlim=c(0.6, 0.95))
plot(mc2sps2, "pB", xlim=c(0.2, 0.95))

diff <- apply(mc2sps2$pA, 1, diff)
postPlot(diff, compVal=0)


# Model with covariates
# =====================

occ2sps_covars.code <- nimbleCode({
  # Likelihood
  for(i in 1:nSites) {
     # Ecological model
     logit(psiA[i]) <- a0 + aArea * area[i]
     zA[i] ~ dbern(psiA[i])
     zB[i] ~ dbern(psiB[(reeds[i]+1)*zA[i] + 1])
     # Observation model
     for(j in 1:n) {
       DHA[i, j] ~ dbern(pA[zB[i] + 1] * zA[i])
       DHB[i, j] ~ dbern(pB[zA[i] + DHA[i, j] + 1] * zB[i])
     }
  }

  # Priors
  pA[1] ~ dbeta(1, 1) # as in previous model
  pA[2] ~ dbeta(1, 1)
  pB[1] ~ dbeta(1, 1)
  pB[2] ~ dbeta(1, 1)
  pB[3] ~ dbeta(1, 1)

  a0 ~ dunif(-5, 5)
  aArea ~ dunif(-5, 5)
  psiB[1] ~ dbeta(1, 1)  # when sps A absent
  psiB[2] ~ dbeta(1, 1)  # when sps A present, no reeds
  psiB[3] ~ dbeta(1, 1)  # when sps A present, reeds
})

nimData <- list(DHA = DHA, DHB = DHB, zA = zA, zB = zB)  # same as before
nimConst <- list(n = 3, nSites = nSites,
    area=c(railSims$logArea), reeds=as.numeric(railSims$reeds))
str(nimConst)

# 'inits' the same

wanted <- c("pA", "pB", "a0", "aArea", "psiB")

system.time(
out3 <- nimbleMCMC(occ2sps_covars.code, nimConst, nimData, inits, monitors=wanted,
    nchains=3, niter=11e3, nburnin=1e3, samplesAsCodaMCMC=TRUE) ) # 65 secs
diagPlot(out3)  # convergence is good

mc2sps3 <- mcmcOutput(out3)
summary(mc2sps3)
#           mean    sd median    l95   u95  Rhat MCEpc
# pA[1]    0.834 0.041  0.837  0.752 0.912 0.998 1.064
# pA[2]    0.738 0.053  0.741  0.630 0.835 0.998 1.184
# pB[1]    0.809 0.029  0.810  0.751 0.862 0.999 0.813
# pB[2]    0.691 0.098  0.696  0.496 0.873 1.001 1.015
# pB[3]    0.345 0.070  0.343  0.211 0.484 0.999 1.409
# a0      -0.150 0.195 -0.150 -0.522 0.243 1.000 0.708
# aArea    1.477 0.266  1.467  0.954 1.998 1.000 0.708
# psiB[1]  0.803 0.044  0.806  0.718 0.888 1.001 0.574
# psiB[2]  0.313 0.086  0.306  0.155 0.487 1.000 1.068
# psiB[3]  0.701 0.116  0.698  0.492 0.950 1.001 1.442

plot(mc2sps3)
plot(mc2sps3, "psiB", xlim=c(0.1, 1))
plot(mc2sps3, "aArea")

# Plot the effect of logArea
# --------------------------
range(railSims$logArea) # +/- 2.5
xx <- seq(-2.5, 2.5, length=101)
toPlot <- matrix(NA, 101, 3)
colnames(toPlot) <- c("est", "lo", "up")
for(i in 1:101){
  psi <- plogis(mc2sps3$a0 + mc2sps3$aArea * xx[i])
  toPlot[i, ] <- c(mean(psi), hdi(psi))
}

plot(xx, toPlot[, 1], type='n', ylim=range(toPlot),
    xlab="standardised log area", ylab="Occupancy")
polygon(c(xx, rev(xx)), c(toPlot[,2], rev(toPlot[,3])),
    col='skyblue', border=NA)
lines(xx, toPlot[,1], lwd=2)
